#include <SoftwareSerial.h>
#include <Wire.h>
#include <avr/sleep.h>
#include "SSD1306Ascii.h"
#include "SSD1306AsciiWire.h"

#define I2C_ADDRESS 0x3C
#define RTN_CHECK 1
#define NUMBER_COUNT 2

SoftwareSerial mySerial(10, 11); // RX, TX
SSD1306AsciiWire oled;
TickerState state;
uint32_t tickTime = 0;
bool doTick = false;

String inputString = "";
bool stringComplete = false;
String simString = "";
bool simStringComplete = false;
bool sleepWaitForOK = false;

unsigned long timer = millis();
unsigned long sleepTimer = timer;
long sleepAfter = 15000;
bool sleepEnable = true;
bool simReady = false;

bool wakeButt = false;
bool callButt = false;
bool stornoButt = false;

struct PhoneBook {
  String name;
  String command;
  String ticketText;
};
PhoneBook n1 = {"A", "ATD+420123456789;", "Calling B "};
PhoneBook n2 = {"B", "ATD+420123456798;", "Calling A "};
PhoneBook phoneBook[2] = {n1, n2};
int currentNumber = -1;

void setup() {
  Serial.begin(9600);
  while (!Serial) {
    ;
  }
  
  mySerial.begin(9600);
  mySerial.println("AT");
  delay(500);
  mySerial.println("AT+CSCLK=0");
  delay(500);
  mySerial.println("AT+CFUN=1");

  Wire.begin();
  Wire.setClock(400000L);
  oled.begin(&Adafruit128x32, I2C_ADDRESS);
  oled.setFont(System5x7);
  
  #if INCLUDE_SCROLLING == 0
  #error INCLUDE_SCROLLING must be non-zero.  Edit SSD1306Ascii.h
  #endif //  INCLUDE_SCROLLING
  // Set auto scrolling at end of window.
  oled.setScrollMode(SCROLL_MODE_AUTO);
  drawTextWithBatt("Preparing...");

  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(2), wakeIntr, FALLING);
  attachInterrupt(digitalPinToInterrupt(3), callIntr, FALLING);
}

void loop() {
  while (mySerial.available()) {
      char inChar = (char) mySerial.read();
        
      if (inChar == '\n') {
          simStringComplete = true;
      } else {
          simString += inChar;
      }
  }

  if(simStringComplete) {
    Serial.println(simString);

    if(simString.indexOf("SMS") >= 0) {
      drawTextWithBatt("SMS READY");
      sleepTimer = millis(); // reset sleep timer
    }

    if(simString.indexOf("Call") >= 0) {
      drawTextWithBatt("CALL READY");
      sleepTimer = millis(); // reset sleep timer
      simReady = true;
    }

    if(simString.indexOf("BUSY") >= 0) {
      resetAfterTickText();
    }

    if(sleepWaitForOK && simString.indexOf("OK") >= 0) {
      sleepWaitForOK = false;
      continueSleep();
    }
    
    simString = "";
    simStringComplete = false;
  }
  
  while (Serial.available()) {
      char inChar = (char) Serial.read();
        
      if (inChar == '\n') {
          stringComplete = true;
      } else {
          inputString += inChar;
      }
  }

  if(stringComplete) {
    if(inputString.indexOf("sleep") >= 0) {
      Serial.println("sleep mode");
      sleepHandler();
    } else if(inputString.indexOf("batt") >= 0) {
      int vcc = readVcc();
      Serial.println(vcc);
      oled.clear();
      oled.print("batt: ");
      oled.println(vcc);
    } else if(inputString.indexOf("dev") >= 0) {
      Serial.println("sleep off");
      sleepEnable = false;
    } else {
      mySerial.println(inputString);
    }
    
    stringComplete = false;
    inputString = "";
  }

  if(sleepEnable && millis() - sleepTimer > sleepAfter) {
    sleepTimer = millis();
    sleepHandler();
  }

  if(wakeButt) { // number change
    Serial.println("wakeButt");
    sleepTimer = millis(); // reset sleep timer

    currentNumber++;
    if(currentNumber >= NUMBER_COUNT) currentNumber = 0;
    
    drawTextWithBatt("-> " + phoneBook[currentNumber].name);
    
    delay(100);
    wakeButt = false;
  }

  if(callButt) {
    Serial.println("callButt");
    if(simReady) {
      sleepEnable = false;
      mySerial.println(phoneBook[currentNumber].command);
      oled.clear();
      oled.tickerInit(&state, Adafruit5x7, 1, true);
      oled.tickerText(&state, phoneBook[currentNumber].ticketText);
      doTick = true;
    }
    
    delay(100);
    callButt = false;
  }

  if(digitalRead(4) == LOW) {
    Serial.println("stornoButt");
    mySerial.println("ATH");
    resetAfterTickText();
    
    delay(100);
    stornoButt = false;
  }

  if(doTick && tickTime <= millis()) {
    tickTime = millis() + 30;

    int8_t rtn = oled.tickerTick(&state);

    if (rtn <= RTN_CHECK) {
      oled.tickerText(&state, phoneBook[currentNumber].ticketText);
    }
  }
}

int readVcc(void) {
  int result;
  
  ADCSRA = (1<<ADEN);  //enable and
  ADCSRA |= (1<<ADPS0) | (1<<ADPS1) | (1<<ADPS2);  // set prescaler to 128
  
  ADMUX = (1<<REFS0) | (1<<MUX3) | (1<<MUX2) | (1<<MUX1);
  delay(1); // Wait for ADC and Vref to settle
  
  ADCSRA |= (1<<ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // wait until done
  result = ADC;
  
  ADCSRA |= (1<<ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // wait until done
  result = ADC;
  
  result = 1140000UL / (unsigned long) result; //1126400 = 1.1*1024*1000
  return result; // Vcc in millivolts
}

void wakeIntr(){
  wakeButt = true;
}

void callIntr(){
  callButt = true;
}

void stornoIntr(){
  stornoButt = true;
}

void sleepHandler() {
  simReady = false;
  Serial.println("sleep mode");
  Serial.flush();
  drawTextWithBatt("sleep");
  mySerial.println("AT+CFUN=0");
  mySerial.flush();

  // potřeba počkat až to vrátí OK
  sleepWaitForOK = true;
}

void continueSleep() {
  Serial.println("sleep continue");
  mySerial.println("AT+CSCLK=2");
  mySerial.flush();
  mySerial.end();
  
  oled.ssd1306WriteCmd(SSD1306_DISPLAYOFF);
  
  set_sleep_mode(SLEEP_MODE_PWR_DOWN); // choose power down mode
  sleep_enable();
  sleep_mode(); // sleep now!

  sleep_disable();

  oled.ssd1306WriteCmd(SSD1306_DISPLAYON);
  drawTextWithBatt("Preparing...");
  mySerial.begin(9600);
  while (!Serial) { }
  mySerial.println("AT");
  delay(500);
  mySerial.println("AT+CSCLK=0");
  delay(500);
  mySerial.println("AT+CFUN=1");
  Serial.println("woke up");

  wakeButt = false;
}

void resetAfterTickText() {
  doTick = false;
  oled.setFont(System5x7);
  drawTextWithBatt("CALL READY");
  sleepEnable = true;
  sleepTimer = millis();
}

void drawTextWithBatt(String text) {
  oled.clear();
  oled.set2X();
  oled.println(text);
  oled.set2X();
  oled.print("Batt: ");
  oled.print(map(readVcc(), 2500, 4200, 0, 100));
  oled.print("%");
}
